@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2001, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2001, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -727,7 +727,7 @@ static int disable_binlog()
 
 static int handle_request_for_tables(char *tables, size_t length)
 {
-  char *query, *end, options[100], message[100];
+  char *query, *end, options[100];
   size_t query_length= 0, query_size= sizeof(char)*(length+110);
   const char *op = 0;
 
@@ -781,7 +781,10 @@ static int handle_request_for_tables(char *tables, size_t length)
   }
   if (mysql_real_query(sock, query, query_length))
   {
-    sprintf(message, "when executing '%s TABLE ... %s'", op, options);
+    static const char format_msg[]= "when executing '%s TABLE ... %s'";
+    char message[sizeof(options) + sizeof(format_msg) +
+                 32 /* to fit "OPTIMIZE NO_WRITE_TO_BINLOG" */];
+    snprintf(message, sizeof(message), format_msg, op, options);
     DBerror(sock, message);
     return 1;
   }
@@ -938,7 +941,7 @@ static int dbConnect(char *host, char *user, char *passwd)
   if (!(sock = mysql_connect_ssl_check(&mysql_connection, host, user, passwd,
                                        NULL, opt_mysql_port,
                                        opt_mysql_unix_port, 0,
-                                       opt_ssl_required)))
+                                       opt_ssl_mode == SSL_MODE_REQUIRED)))
   {
     DBerror(&mysql_connection, "when trying to connect");
     DBUG_RETURN(1);
