@@ -80,8 +80,8 @@ extern "C" {
 
 #define mi_is_key_active(_keymap_,_keyno_) \
                             (((_keyno_) < MI_KEYMAP_BITS) ? \
-                             test((_keymap_) & (ULL(1) << (_keyno_))) : \
-                             test((_keymap_) & MI_KEYMAP_HIGH_MASK))
+                             MY_TEST((_keymap_) & (ULL(1) << (_keyno_))) : \
+                             MY_TEST((_keymap_) & MI_KEYMAP_HIGH_MASK))
 #define mi_set_key_active(_keymap_,_keyno_) \
                             (_keymap_)|= (((_keyno_) < MI_KEYMAP_BITS) ? \
                                           (ULL(1) << (_keyno_)) : \
@@ -94,7 +94,7 @@ extern "C" {
 #else
 
 #define mi_is_key_active(_keymap_,_keyno_) \
-                            test((_keymap_) & (ULL(1) << (_keyno_)))
+                            MY_TEST((_keymap_) & (ULL(1) << (_keyno_)))
 #define mi_set_key_active(_keymap_,_keyno_) \
                             (_keymap_)|= (ULL(1) << (_keyno_))
 #define mi_clear_key_active(_keymap_,_keyno_) \
@@ -103,7 +103,7 @@ extern "C" {
 #endif
 
 #define mi_is_any_key_active(_keymap_) \
-                            test((_keymap_))
+                            MY_TEST((_keymap_))
 #define mi_is_all_keys_active(_keymap_,_keys_) \
                             ((_keymap_) == mi_get_mask_all_keys_active(_keys_))
 #define mi_set_all_keys_active(_keymap_,_keys_) \
@@ -119,7 +119,7 @@ extern "C" {
                             (_to_)= (mi_get_mask_all_keys_active(_maxkeys_) & \
                                      (_from_))
 
-	/* Param to/from mi_info */
+	/* Param to/from mi_status */
 
 typedef struct st_mi_isaminfo		/* Struct from h_info */
 {
