@@ -41,6 +41,7 @@
 #define TABLE_COMMENT_MAXLEN 2048
 #define COLUMN_COMMENT_MAXLEN 1024
 #define INDEX_COMMENT_MAXLEN 1024
+#define TABLE_PARTITION_COMMENT_MAXLEN 1024
 
 /*
   USER_HOST_BUFF_SIZE -- length of string buffer, that is enough to contain
@@ -73,6 +74,7 @@ enum enum_server_command
   COM_TABLE_DUMP, COM_CONNECT_OUT, COM_REGISTER_SLAVE,
   COM_STMT_PREPARE, COM_STMT_EXECUTE, COM_STMT_SEND_LONG_DATA, COM_STMT_CLOSE,
   COM_STMT_RESET, COM_SET_OPTION, COM_STMT_FETCH, COM_DAEMON,
+  COM_BINLOG_DUMP_GTID,
   /* don't forget to update const char *command_name[] in sql_parse.cc */
 
   /* Must be last */
@@ -114,12 +116,19 @@ enum enum_server_command
 #define BINCMP_FLAG	131072		/* Intern: Used by sql_yacc */
 #define GET_FIXED_FIELDS_FLAG (1 << 18) /* Used to get fields in item tree */
 #define FIELD_IN_PART_FUNC_FLAG (1 << 19)/* Field part of partition func */
-#define FIELD_IN_ADD_INDEX (1<< 20)	/* Intern: Field used in ADD INDEX */
+/**
+  Intern: Field in TABLE object for new version of altered table,
+          which participates in a newly added index.
+*/
+#define FIELD_IN_ADD_INDEX (1 << 20)
 #define FIELD_IS_RENAMED (1<< 21)       /* Intern: Field is being renamed */
-#define FIELD_FLAGS_STORAGE_MEDIA 22    /* Field storage media, bit 22-23,
-                                           reserved by MySQL Cluster */
-#define FIELD_FLAGS_COLUMN_FORMAT 24    /* Field column format, bit 24-25,
-                                           reserved by MySQL Cluster */
+#define FIELD_FLAGS_STORAGE_MEDIA 22    /* Field storage media, bit 22-23 */
+#define FIELD_FLAGS_STORAGE_MEDIA_MASK (3 << FIELD_FLAGS_STORAGE_MEDIA)
+#define FIELD_FLAGS_COLUMN_FORMAT 24    /* Field column format, bit 24-25 */
+#define FIELD_FLAGS_COLUMN_FORMAT_MASK (3 << FIELD_FLAGS_COLUMN_FORMAT)
+#define FIELD_IS_DROPPED (1<< 26)       /* Intern: Field is being dropped */
+#define CLUSTERING_FLAG (1 << 27)       /* Field has a secondary clustering
+                                        key */
 
 #define REFRESH_GRANT		1	/* Refresh grant tables */
 #define REFRESH_LOG		2	/* Start on new log file */
@@ -143,16 +152,21 @@ enum enum_server_command
 #define REFRESH_FAST		32768	/* Intern flag */
 
 /* RESET (remove all queries) from query cache */
-#define REFRESH_QUERY_CACHE	     65536
-#define REFRESH_QUERY_CACHE_FREE    0x20000L /* pack query cache */
-#define REFRESH_DES_KEY_FILE	     0x40000L
-#define REFRESH_USER_RESOURCES	     0x80000L
-#define REFRESH_QUERY_RESPONSE_TIME 0x100000L /* response time distibution */
-#define REFRESH_TABLE_STATS    0x200000L /* Refresh table stats my_hash table */
-#define REFRESH_INDEX_STATS    0x400000L /* Refresh index stats my_hash table */
-#define REFRESH_USER_STATS     0x800000L /* Refresh user stats my_hash table */
-#define REFRESH_CLIENT_STATS   0x1000000L /* Refresh client stats my_hash table */
-#define REFRESH_THREAD_STATS   0x2000000L /* Refresh thread stats my_hash table */
+#define REFRESH_QUERY_CACHE	65536
+#define REFRESH_QUERY_CACHE_FREE 0x20000L /* pack query cache */
+#define REFRESH_DES_KEY_FILE	0x40000L
+#define REFRESH_USER_RESOURCES	0x80000L
+#define REFRESH_FOR_EXPORT      0x100000L /* FLUSH TABLES ... FOR EXPORT */
+#define REFRESH_TABLE_STATS     0x200000L /* Refresh table stats my_hash
+                                             table */
+#define REFRESH_INDEX_STATS     0x400000L /* Refresh index stats my_hash
+                                             table */
+#define REFRESH_USER_STATS      0x800000L /* Refresh user stats my_hash
+                                             table */
+#define REFRESH_CLIENT_STATS    0x1000000L /* Refresh client stats my_hash
+                                              table */
+#define REFRESH_THREAD_STATS    0x2000000L /* Refresh thread stats my_hash
+                                              table */
 #define REFRESH_FLUSH_PAGE_BITMAPS 0x4000000L
 #define REFRESH_RESET_PAGE_BITMAPS 0x8000000L
 
@@ -179,6 +193,13 @@ enum enum_server_command
 #define CLIENT_PS_MULTI_RESULTS (1UL << 18) /* Multi-results in PS-protocol */
 
 #define CLIENT_PLUGIN_AUTH  (1UL << 19) /* Client supports plugin authentication */
+#define CLIENT_CONNECT_ATTRS (1UL << 20) /* Client supports connection attributes */
+
+/* Enable authentication response packet to be larger than 255 bytes. */
+#define CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA (1UL << 21)
+
+/* Don't close the connection for a connection with expired password. */
+#define CLIENT_CAN_HANDLE_EXPIRED_PASSWORDS (1UL << 22)
 
 #define CLIENT_SSL_VERIFY_SERVER_CERT (1UL << 30)
 #define CLIENT_REMEMBER_OPTIONS (1UL << 31)
@@ -190,28 +211,32 @@ enum enum_server_command
 #endif
 
 /* Gather all possible capabilites (flags) supported by the server */
-#define CLIENT_ALL_FLAGS  (CLIENT_LONG_PASSWORD | \
-                           CLIENT_FOUND_ROWS | \
-                           CLIENT_LONG_FLAG | \
-                           CLIENT_CONNECT_WITH_DB | \
-                           CLIENT_NO_SCHEMA | \
-                           CLIENT_COMPRESS | \
-                           CLIENT_ODBC | \
-                           CLIENT_LOCAL_FILES | \
-                           CLIENT_IGNORE_SPACE | \
-                           CLIENT_PROTOCOL_41 | \
-                           CLIENT_INTERACTIVE | \
-                           CLIENT_SSL | \
-                           CLIENT_IGNORE_SIGPIPE | \
-                           CLIENT_TRANSACTIONS | \
-                           CLIENT_RESERVED | \
-                           CLIENT_SECURE_CONNECTION | \
-                           CLIENT_MULTI_STATEMENTS | \
-                           CLIENT_MULTI_RESULTS | \
-                           CLIENT_PS_MULTI_RESULTS | \
-                           CLIENT_SSL_VERIFY_SERVER_CERT | \
-                           CLIENT_REMEMBER_OPTIONS | \
-                           CLIENT_PLUGIN_AUTH)
+#define CLIENT_ALL_FLAGS  (CLIENT_LONG_PASSWORD \
+                           | CLIENT_FOUND_ROWS \
+                           | CLIENT_LONG_FLAG \
+                           | CLIENT_CONNECT_WITH_DB \
+                           | CLIENT_NO_SCHEMA \
+                           | CLIENT_COMPRESS \
+                           | CLIENT_ODBC \
+                           | CLIENT_LOCAL_FILES \
+                           | CLIENT_IGNORE_SPACE \
+                           | CLIENT_PROTOCOL_41 \
+                           | CLIENT_INTERACTIVE \
+                           | CLIENT_SSL \
+                           | CLIENT_IGNORE_SIGPIPE \
+                           | CLIENT_TRANSACTIONS \
+                           | CLIENT_RESERVED \
+                           | CLIENT_SECURE_CONNECTION \
+                           | CLIENT_MULTI_STATEMENTS \
+                           | CLIENT_MULTI_RESULTS \
+                           | CLIENT_PS_MULTI_RESULTS \
+                           | CLIENT_SSL_VERIFY_SERVER_CERT \
+                           | CLIENT_REMEMBER_OPTIONS \
+                           | CLIENT_PLUGIN_AUTH \
+                           | CLIENT_CONNECT_ATTRS \
+                           | CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA \
+                           | CLIENT_CAN_HANDLE_EXPIRED_PASSWORDS \
+)
 
 /*
   Switch off the flags that are optional and depending on build flags
@@ -260,6 +285,16 @@ enum enum_server_command
 */
 #define SERVER_PS_OUT_PARAMS            4096
 
+/**
+  Set at the same time as SERVER_STATUS_IN_TRANS if the started
+  multi-statement transaction is a read-only transaction. Cleared
+  when the transaction commits or aborts. Since this flag is sent
+  to clients in OK and EOF packets, the flag indicates the
+  transaction status at the end of command execution.
+*/
+#define SERVER_STATUS_IN_TRANS_READONLY 8192
+
+
 /**
   Server status flags that must be cleared when starting
   execution of a new SQL statement.
@@ -314,7 +349,7 @@ typedef struct st_net {
   unsigned int *return_status;
   unsigned char reading_or_writing;
   char save_char;
-  my_bool unused1; /* Please remove with the next incompatible ABI change. */
+  my_bool unused1; /* Please remove with the next incompatible ABI change */
   my_bool unused2; /* Please remove with the next incompatible ABI change */
   my_bool compress;
   my_bool unused3; /* Please remove with the next incompatible ABI change. */
@@ -335,17 +370,15 @@ typedef struct st_net {
   char last_error[MYSQL_ERRMSG_SIZE];
   /** Client library sqlstate buffer. Set along with the error message. */
   char sqlstate[SQLSTATE_LENGTH+1];
-  void *extension;
-#if defined(MYSQL_SERVER) && !defined(EMBEDDED_LIBRARY)
-  /*
-    Controls whether a big packet should be skipped.
-
-    Initially set to FALSE by default. Unauthenticated sessions must have
-    this set to FALSE so that the server can't be tricked to read packets
-    indefinitely.
+  /**
+    Extension pointer, for the caller private use.
+    Any program linking with the networking library can use this pointer,
+    which is handy when private connection specific data needs to be
+    maintained.
+    The mysqld server process uses this pointer internally,
+    to maintain the server internal instrumentation for the connection.
   */
-  my_bool skip_big_packet;
-#endif
+  void *extension;
 } NET;
 
 
@@ -360,6 +393,9 @@ enum enum_field_types { MYSQL_TYPE_DECIMAL, MYSQL_TYPE_TINY,
 			MYSQL_TYPE_DATETIME, MYSQL_TYPE_YEAR,
 			MYSQL_TYPE_NEWDATE, MYSQL_TYPE_VARCHAR,
 			MYSQL_TYPE_BIT,
+			MYSQL_TYPE_TIMESTAMP2,
+			MYSQL_TYPE_DATETIME2,
+			MYSQL_TYPE_TIME2,
                         MYSQL_TYPE_NEWDECIMAL=246,
 			MYSQL_TYPE_ENUM=247,
 			MYSQL_TYPE_SET=248,
@@ -461,27 +497,23 @@ extern "C" {
 #endif
 
 my_bool	my_net_init(NET *net, Vio* vio);
-void	my_net_local_init(NET *net);
-void	net_end(NET *net);
-  void	net_clear(NET *net, my_bool clear_buffer);
+void my_net_local_init(NET *net);
+void net_end(NET *net);
+void net_clear(NET *net, my_bool check_buffer);
 my_bool net_realloc(NET *net, size_t length);
 my_bool	net_flush(NET *net);
 my_bool	my_net_write(NET *net,const unsigned char *packet, size_t len);
 my_bool	net_write_command(NET *net,unsigned char command,
 			  const unsigned char *header, size_t head_len,
 			  const unsigned char *packet, size_t len);
-int	net_real_write(NET *net,const unsigned char *packet, size_t len);
+my_bool net_write_packet(NET *net, const unsigned char *packet, size_t length);
 unsigned long my_net_read(NET *net);
 
-#ifdef _global_h
+#ifdef MY_GLOBAL_INCLUDED
 void my_net_set_write_timeout(NET *net, uint timeout);
 void my_net_set_read_timeout(NET *net, uint timeout);
 #endif
 
-struct sockaddr;
-int my_connect(my_socket s, const struct sockaddr *name, unsigned int namelen,
-	       unsigned int timeout);
-
 struct rand_struct {
   unsigned long seed1,seed2,max_value;
   double max_value_dbl;
@@ -570,7 +602,7 @@ const char *mysql_errno_to_sqlstate(unsigned int mysql_errno);
 my_bool my_thread_init(void);
 void my_thread_end(void);
 
-#ifdef _global_h
+#ifdef MY_GLOBAL_INCLUDED
 ulong STDCALL net_field_length(uchar **packet);
 ulong STDCALL net_field_length_checked(uchar **packet, ulong max_length);
 my_ulonglong net_field_length_ll(uchar **packet);
