@@ -475,8 +475,8 @@ else
   # layout is used
 
   cd "`dirname $0`"
-  if [ -h "$0" ] ; then
-    realpath="`ls -l  "$0" | awk '{print $NF}'`"
+  if [ -h "`dirname $0`" -o -h "$0" ] ; then
+    realpath=$(readlink -f "$0")
     cd "`dirname "$realpath"`"
   fi
   cd ..
