@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ extern TYPELIB binlog_checksum_typelib;
 
 extern "C" uint32
 *slave_list_key(SLAVE_INFO* si, size_t *len,
-		my_bool not_used __attribute__((unused)))
+		my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *len = 4;
   return &si->server_id;
@@ -738,7 +738,6 @@ bool com_binlog_dump(THD *thd, char *packet, uint packet_length)
 {
   DBUG_ENTER("com_binlog_dump");
   ulong pos;
-  String slave_uuid;
   ushort flags= 0;
   const uchar* packet_position= (uchar *) packet;
   uint packet_bytes_todo= packet_length;
@@ -759,8 +758,7 @@ bool com_binlog_dump(THD *thd, char *packet, uint packet_length)
 
   DBUG_PRINT("info", ("pos=%lu flags=%d server_id=%d", pos, flags, thd->server_id));
 
-  get_slave_uuid(thd, &slave_uuid);
-  kill_zombie_dump_threads(&slave_uuid);
+  kill_zombie_dump_threads(thd);
 
   general_log_print(thd, thd->get_command(), "Log: '%s'  Pos: %ld",
                     packet + 10, (long) pos);
@@ -783,7 +781,6 @@ bool com_binlog_dump_gtid(THD *thd, char *packet, uint packet_length)
     Before going GA, we need to make this protocol extensible without
     breaking compatitibilty. /Alfranio.
   */
-  String slave_uuid;
   ushort flags= 0;
   uint32 data_size= 0;
   uint64 pos= 0;
@@ -815,8 +812,7 @@ bool com_binlog_dump_gtid(THD *thd, char *packet, uint packet_length)
   DBUG_PRINT("info", ("Slave %d requested to read %s at position %llu gtid set "
                       "'%s'.", thd->server_id, name, pos, gtid_string));
 
-  get_slave_uuid(thd, &slave_uuid);
-  kill_zombie_dump_threads(&slave_uuid);
+  kill_zombie_dump_threads(thd);
   general_log_print(thd, thd->get_command(), "Log: '%s' Pos: %llu GTIDs: '%s'",
                     name, pos, gtid_string);
   my_free(gtid_string);
@@ -849,6 +845,7 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
   char *log_file_name = linfo.log_file_name;
   char search_file_name[FN_REFLEN], *name;
 
+  bool is_index_file_reopened_on_binlog_disable= false;
   ulong ev_offset;
   bool using_gtid_protocol= slave_gtid_executed != NULL;
   bool searching_first_gtid= using_gtid_protocol;
@@ -889,6 +886,7 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
   Diagnostics_area temp_da;
   Diagnostics_area *saved_da= thd->get_stmt_da();
   thd->set_stmt_da(&temp_da);
+  bool was_killed_by_duplicate_slave_id= false;
 
   DBUG_ENTER("mysql_binlog_send");
   DBUG_PRINT("enter",("log_ident: '%s'  pos: %ld", log_ident, (long) pos));
@@ -1249,6 +1247,10 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
                                               log_file_name,
                                               &is_active_binlog)))
     {
+      DBUG_EXECUTE_IF("simulate_dump_thread_kill",
+                      {
+                        thd->killed= THD::KILL_CONNECTION;
+                      });
       DBUG_EXECUTE_IF("hold_dump_thread_inside_inner_loop",
                     {
                       const char act[]= "now "
@@ -1557,6 +1559,10 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
         GOTO_ERR;
     }
 
+    /* If the above while is killed due to thd->killed flag and not
+      due to read_log_event error, then do nothing.*/
+    if (thd->killed)
+      goto end;
     DBUG_EXECUTE_IF("wait_after_binlog_EOF",
                     {
                       const char act[]= "now wait_for signal.rotate_finished no_clear_event";
@@ -1930,18 +1936,65 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
 
     if (goto_next_binlog)
     {
+      DBUG_EXECUTE_IF("waiting_for_disable_binlog",
+                      {
+                      const char act[]= "now "
+                      "signal dump_thread_reached_wait_point "
+                      "wait_for continue_dump_thread no_clear_event";
+                      DBUG_ASSERT(!debug_sync_set_action(current_thd,
+                                                         STRING_WITH_LEN(act)));
+                      };);
+
       // clear flag because we open a new binlog
       binlog_has_previous_gtids_log_event= false;
 
       THD_STAGE_INFO(thd, stage_finished_reading_one_binlog_switching_to_next_binlog);
-      switch (mysql_bin_log.find_next_log(&linfo, 1)) {
-      case 0:
-        break;
-      default:
+      /*
+        When moving to next binlog, always check if binlog is enabled or not.
+        There can be critical errors like for example out of memory scenarios
+        which prevent mysqld server from writing to binlog. In those cases
+        server refers to binlog_error_action variable and takes the appropriate
+        action. If users choose to ignore the error then binary log will be
+        disabled and server will continue to do its work. In such cases dump
+        thread which is trying to move to the next log will fail as binlog index
+        file and binlog file are already closed and their corresponding caches
+        are also cleared.
+
+        Hence first check if binary log is enabled or not. If enabled look for
+        the next binary log in the index file. If it is disabled open the index
+        file once again and check if there any more binary logs that needs to be
+        sent. Keep reading binary log files until find_next_log returns empty.
+        If there is an error during open index file or we sent all binary logs
+        then ER_MASTER_FATAL_ERROR_READING_BINLOG is raised.
+      */
+      mysql_bin_log.lock_index();
+      if (!mysql_bin_log.is_open())
+      {
+        if (mysql_bin_log.open_index_file(mysql_bin_log.get_index_fname(),
+                                          linfo.log_file_name,FALSE))
+        {
+          errmsg = "Binary log is not open and failed to open index file to "
+            "retrieve next file.";
+          my_errno= ER_MASTER_FATAL_ERROR_READING_BINLOG;
+          mysql_bin_log.unlock_index();
+          GOTO_ERR;
+        }
+        is_index_file_reopened_on_binlog_disable= true;
+      }
+      if (mysql_bin_log.find_next_log(&linfo, 0))
+      {
+        DBUG_EXECUTE_IF("waiting_for_disable_binlog",
+                        {
+                        const char act[]= "now signal consumed_binlog";
+                        DBUG_ASSERT(!debug_sync_set_action(current_thd,
+                                                           STRING_WITH_LEN(act)));
+                        };);
         errmsg = "could not find next log";
         my_errno= ER_MASTER_FATAL_ERROR_READING_BINLOG;
+        mysql_bin_log.unlock_index();
         GOTO_ERR;
       }
+      mysql_bin_log.unlock_index();
 
       end_io_cache(&log);
       mysql_file_close(file, MYF(MY_WME));
@@ -1980,6 +2033,21 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
   }
 
 end:
+  /*
+    If the dump thread was killed because of a duplicate slave UUID we
+    will fail throwing an error to the slave so it will not try to
+    reconnect anymore.
+  */
+  mysql_mutex_lock(&thd->LOCK_thd_data);
+  was_killed_by_duplicate_slave_id= thd->duplicate_slave_id;
+  mysql_mutex_unlock(&thd->LOCK_thd_data);
+  if (was_killed_by_duplicate_slave_id)
+  {
+    errmsg= "A slave with the same server_uuid/server_id as this slave "
+            "has connected to the master";
+    my_errno= ER_MASTER_FATAL_ERROR_READING_BINLOG;
+    goto err;
+  }
   thd->set_stmt_da(saved_da);
   end_io_cache(&log);
   mysql_file_close(file, MYF(MY_WME));
@@ -2017,6 +2085,9 @@ err:
     error_text[sizeof(error_text) - 1]= '\0';
   }
   end_io_cache(&log);
+  if (is_index_file_reopened_on_binlog_disable)
+    mysql_bin_log.close(LOG_CLOSE_INDEX, true/*need_lock_log=true*/,
+			true/*need_lock_index=true*/);
   if (has_transmit_started)
     (void) RUN_HOOK(binlog_transmit, transmit_stop, (thd, flags));
   /*
@@ -2067,41 +2138,58 @@ String *get_slave_uuid(THD *thd, String *value)
 /*
 
   Kill all Binlog_dump threads which previously talked to the same slave
-  ("same" means with the same server id). Indeed, if the slave stops, if the
+  ("same" means with the same UUID(for slave versions >= 5.6) or same server id
+  (for slave versions < 5.6). Indeed, if the slave stops, if the
   Binlog_dump thread is waiting (mysql_cond_wait) for binlog update, then it
   will keep existing until a query is written to the binlog. If the master is
   idle, then this could last long, and if the slave reconnects, we could have 2
   Binlog_dump threads in SHOW PROCESSLIST, until a query is written to the
   binlog. To avoid this, when the slave reconnects and sends COM_BINLOG_DUMP,
-  the master kills any existing thread with the slave's server id (if this id is
+  the master kills any existing thread with the slave's UUID/server id (if this id is
   not zero; it will be true for real slaves, but false for mysqlbinlog when it
   sends COM_BINLOG_DUMP to get a remote binlog dump).
 
   SYNOPSIS
     kill_zombie_dump_threads()
-    slave_uuid      the slave's UUID
+    @param thd newly connected dump thread object
 
 */
 
-
-void kill_zombie_dump_threads(String *slave_uuid)
+void kill_zombie_dump_threads(THD *thd)
 {
-  if (slave_uuid->length() == 0)
+  String slave_uuid;
+  get_slave_uuid(thd, &slave_uuid);
+  if (slave_uuid.length() == 0 && thd->server_id == 0)
     return;
-  DBUG_ASSERT(slave_uuid->length() == UUID_LENGTH);
 
   mysql_mutex_lock(&LOCK_thread_count);
   THD *tmp= NULL;
   Thread_iterator it= global_thread_list_begin();
   Thread_iterator end= global_thread_list_end();
+  bool is_zombie_thread= false;
   for (; it != end; ++it)
   {
-    if ((*it) != current_thd && ((*it)->get_command() == COM_BINLOG_DUMP ||
-                                 (*it)->get_command() == COM_BINLOG_DUMP_GTID))
+    if ((*it) != thd && ((*it)->get_command() == COM_BINLOG_DUMP ||
+                         (*it)->get_command() == COM_BINLOG_DUMP_GTID))
     {
       String tmp_uuid;
-      if (get_slave_uuid((*it), &tmp_uuid) != NULL &&
-          !strncmp(slave_uuid->c_ptr(), tmp_uuid.c_ptr(), UUID_LENGTH))
+      get_slave_uuid((*it), &tmp_uuid);
+      if (slave_uuid.length())
+      {
+        is_zombie_thread= (tmp_uuid.length() &&
+                           !strncmp(slave_uuid.c_ptr(),
+                                    tmp_uuid.c_ptr(), UUID_LENGTH));
+      }
+      else
+      {
+        /*
+          Check if it is a 5.5 slave's dump thread i.e., server_id should be
+          same && dump thread should not contain 'UUID'.
+        */
+        is_zombie_thread= (((*it)->server_id == thd->server_id) &&
+                           !tmp_uuid.length());
+      }
+      if (is_zombie_thread)
       {
         tmp= *it;
         mysql_mutex_lock(&tmp->LOCK_thd_data);	// Lock from delete
@@ -2118,16 +2206,30 @@ void kill_zombie_dump_threads(String *slave_uuid)
       again. We just to do kill the thread ourselves.
     */
     if (log_warnings > 1)
-      sql_print_information("While initializing dump thread for slave with "
-                            "UUID <%s>, found a zombie dump thread with "
-                            "the same UUID. Master is killing the zombie dump "
-                            "thread(%lu).", slave_uuid->c_ptr(), tmp->thread_id);
+    {
+      if (slave_uuid.length())
+      {
+        sql_print_information("While initializing dump thread for slave with "
+                              "UUID <%s>, found a zombie dump thread with the "
+                              "same UUID. Master is killing the zombie dump "
+                              "thread(%lu).", slave_uuid.c_ptr(),
+                              tmp->thread_id);
+      }
+      else
+      {
+        sql_print_information("While initializing dump thread for slave with "
+                              "server_id <%u>, found a zombie dump thread with the "
+                              "same server_id. Master is killing the zombie dump "
+                              "thread(%lu).", thd->server_id,
+                              tmp->thread_id);
+      }
+    }
+    tmp->duplicate_slave_id= true;
     tmp->awake(THD::KILL_QUERY);
     mysql_mutex_unlock(&tmp->LOCK_thd_data);
   }
 }
 
-
 /**
   Execute a RESET MASTER statement.
 
