@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -186,6 +186,13 @@ public:
   */
   uint32 cur_log_old_open_count;
 
+  /*
+    If on init_info() call error_on_rli_init_info is true that means
+    that previous call to init_info() terminated with an error, RESET
+    SLAVE must be executed and the problem fixed manually.
+   */
+  bool error_on_rli_init_info;
+
   /*
     Let's call a group (of events) :
       - a transaction
@@ -204,6 +211,11 @@ public:
     happen when, for example, the relay log gets rotated because of
     max_binlog_size.
   */
+
+  // overridden new and delete operators for 64 byte alignment
+  static void* operator new(size_t request);
+  static void operator delete(void * ptr);
+
 protected:
   char group_relay_log_name[FN_REFLEN];
   ulonglong group_relay_log_pos;
@@ -261,7 +273,7 @@ public:
   int init_relay_log_pos(const char* log,
                          ulonglong pos, bool need_data_lock,
                          const char** errmsg,
-                         bool look_for_description_event);
+                         bool keep_looking_for_fd);
 
   /*
     Handling of the relay_log_space_limit optional constraint.
@@ -314,10 +326,7 @@ public:
    */
   enum {UNTIL_NONE= 0, UNTIL_MASTER_POS, UNTIL_RELAY_POS,
         UNTIL_SQL_BEFORE_GTIDS, UNTIL_SQL_AFTER_GTIDS,
-        UNTIL_SQL_AFTER_MTS_GAPS
-#ifndef DBUG_OFF
-        , UNTIL_DONE
-#endif
+        UNTIL_SQL_AFTER_MTS_GAPS, UNTIL_DONE
 }
     until_condition;
   char until_log_name[FN_REFLEN];
@@ -409,8 +418,8 @@ public:
                               bool need_data_lock);
 
   int wait_for_pos(THD* thd, String* log_name, longlong log_pos, 
-		   longlong timeout);
-  int wait_for_gtid_set(THD* thd, String* gtid, longlong timeout);
+		   double timeout);
+  int wait_for_gtid_set(THD* thd, String* gtid, double timeout);
   void close_temporary_tables();
 
   /* Check if UNTIL condition is satisfied. See slave.cc for more. */
@@ -646,7 +655,8 @@ public:
      Coordinator notifies Workers about this event. Coordinator and Workers
      maintain a bitmap of executed group that is reset with a new checkpoint. 
   */
-  void reset_notified_checkpoint(ulong, time_t, bool);
+  void reset_notified_checkpoint(ulong, const time_t *const new_ts,
+                                 bool);
 
   /**
      Called when gaps execution is ended so it is crash-safe
@@ -858,6 +868,11 @@ public:
   */
   bool reported_unsafe_warning;
 
+  /*
+    'sql_thread_kill_accepted is set to TRUE when killed status is recognized.
+  */
+  bool sql_thread_kill_accepted;
+
   time_t get_row_stmt_start_timestamp()
   {
     return row_stmt_start_timestamp;
@@ -911,6 +926,17 @@ public:
   */
   void adapt_to_master_version(Format_description_log_event *fdle);
   uchar slave_version_split[3]; // bytes of the slave server version
+  /*
+    relay log info repository should be updated on relay log
+    rotate. But when the transaction is split across two relay logs,
+    update the repository will cause unexpected results and should
+    be postponed till the 'commit' of the transaction is executed.
+
+    A flag that set to 'true' when this type of 'forced flush'(at the
+    time of rotate relay log) is postponed due to transaction split
+    across the relay logs.
+  */
+  bool force_flush_postponed_due_to_split_trans;
 
 protected:
   Format_description_log_event *rli_description_event;
@@ -972,12 +998,6 @@ private:
   time_t row_stmt_start_timestamp;
   bool long_find_row_note_printed;
 
-  /*
-    If on init_info() call error_on_rli_init_info is true that means
-    that previous call to init_info() terminated with an error, RESET
-    SLAVE must be executed and the problem fixed manually.
-   */
-  bool error_on_rli_init_info;
 };
 
 bool mysql_show_relaylog_events(THD* thd);
