@@ -1,4 +1,5 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights
+ * reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +16,6 @@
 
 /* This file is originally from the mysql distribution. Coded by monty */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #include <my_global.h>
 #include <my_sys.h>
 #include <m_string.h>
@@ -27,6 +24,11 @@
 
 #include "sql_string.h"
 
+#include <algorithm>
+
+using std::min;
+using std::max;
+
 /*****************************************************************************
 ** String functions
 *****************************************************************************/
@@ -63,14 +65,20 @@ bool String::real_alloc(uint32 length)
      - If the requested length is less or equal to what fits in the buffer, a
        null character is inserted at the appropriate position.
 
-   - If the String does not keep a private buffer on the heap, such a buffer
-     will be allocated and the string copied accoring to its length, as found
-     in String::length().
+   - If the String does not keep a private buffer on the heap:
+
+      - If the requested length is greater than what fits in the buffer, or
+        force_on_heap is true, a new buffer is allocated, data is copied.
+      - If the requested length is less or equal to what fits in the buffer,
+        and force_on_heap is false, a null character is inserted at the
+        appropriate position.
  
    For C compatibility, the new string buffer is null terminated.
 
    @param alloc_length The requested string size in characters, excluding any
    null terminator.
+   @param force_on_heap If the caller wants String's 'str' buffer to be on the
+   heap in all cases.
 
    @retval false Either the copy operation is complete or, if the size of the
    new buffer is smaller than the currently allocated buffer (if one exists),
@@ -78,12 +86,19 @@ bool String::real_alloc(uint32 length)
 
    @retval true An error occured when attempting to allocate memory.
 */
-bool String::realloc(uint32 alloc_length)
+bool String::realloc(uint32 alloc_length, bool force_on_heap)
 {
   uint32 len=ALIGN_SIZE(alloc_length+1);
   DBUG_ASSERT(len > alloc_length);
   if (len <= alloc_length)
     return TRUE;                                 /* Overflow */
+
+  if (force_on_heap && !alloced)
+  {
+    /* Bytes will be allocated on the heap.*/
+    Alloced_length= 0;
+  }
+
   if (Alloced_length < len)
   {
     char *new_ptr;
@@ -110,7 +125,7 @@ bool String::realloc(uint32 alloc_length)
   return FALSE;
 }
 
-bool String::set_int(longlong num, bool unsigned_flag, CHARSET_INFO *cs)
+bool String::set_int(longlong num, bool unsigned_flag, const CHARSET_INFO *cs)
 {
   uint l=20*cs->mbmaxlen+1;
   int base= unsigned_flag ? 10 : -10;
@@ -122,7 +137,7 @@ bool String::set_int(longlong num, bool unsigned_flag, CHARSET_INFO *cs)
   return FALSE;
 }
 
-bool String::set_real(double num,uint decimals, CHARSET_INFO *cs)
+bool String::set_real(double num,uint decimals, const CHARSET_INFO *cs)
 {
   char buff[FLOATING_POINT_BUFFER];
   uint dummy_errors;
@@ -172,7 +187,8 @@ bool String::copy(const String &str)
   return FALSE;
 }
 
-bool String::copy(const char *str,uint32 arg_length, CHARSET_INFO *cs)
+bool String::copy(const char *str,uint32 arg_length,
+                  const CHARSET_INFO *cs)
 {
   if (alloc(arg_length))
     return TRUE;
@@ -207,8 +223,8 @@ bool String::copy(const char *str,uint32 arg_length, CHARSET_INFO *cs)
 */
 
 bool String::needs_conversion(uint32 arg_length,
-			      CHARSET_INFO *from_cs,
-			      CHARSET_INFO *to_cs,
+			      const CHARSET_INFO *from_cs,
+			      const CHARSET_INFO *to_cs,
 			      uint32 *offset)
 {
   *offset= 0;
@@ -236,8 +252,8 @@ bool String::needs_conversion(uint32 arg_length,
   @return conversion needed
 */
 bool String::needs_conversion_on_storage(uint32 arg_length,
-                                         CHARSET_INFO *cs_from,
-                                         CHARSET_INFO *cs_to)
+                                         const CHARSET_INFO *cs_from,
+                                         const CHARSET_INFO *cs_to)
 {
   uint32 offset;
   return (needs_conversion(arg_length, cs_from, cs_to, &offset) ||
@@ -285,7 +301,7 @@ bool String::needs_conversion_on_storage(uint32 arg_length,
 */
 
 bool String::copy_aligned(const char *str,uint32 arg_length, uint32 offset,
-			  CHARSET_INFO *cs)
+			  const CHARSET_INFO *cs)
 {
   /* How many bytes are in incomplete character */
   offset= cs->mbminlen - offset; /* How many zeros we should prepend */
@@ -300,7 +316,7 @@ bool String::copy_aligned(const char *str,uint32 arg_length, uint32 offset,
     If we add little-endian UCS-2 sometimes, this code
     will be more complicated. But it's OK for now.
   */
-  bzero((char*) Ptr, offset);
+  memset(Ptr, 0, offset);
   memcpy(Ptr + offset, str, arg_length);
   Ptr[aligned_length]=0;
   /* str_length is always >= 0 as arg_length is != 0 */
@@ -311,7 +327,7 @@ bool String::copy_aligned(const char *str,uint32 arg_length, uint32 offset,
 
 
 bool String::set_or_copy_aligned(const char *str,uint32 arg_length,
-				 CHARSET_INFO *cs)
+				 const CHARSET_INFO *cs)
 {
   /* How many bytes are in incomplete character */
   uint32 offset= (arg_length % cs->mbminlen); 
@@ -336,7 +352,7 @@ bool String::set_or_copy_aligned(const char *str,uint32 arg_length,
 */
 
 bool String::copy(const char *str, uint32 arg_length,
-		  CHARSET_INFO *from_cs, CHARSET_INFO *to_cs, uint *errors)
+		  const CHARSET_INFO *from_cs, const CHARSET_INFO *to_cs, uint *errors)
 {
   uint32 offset;
 
@@ -403,7 +419,7 @@ bool String::fill(uint32 max_length,char fill_char)
   {
     if (realloc(max_length))
       return TRUE;
-    bfill(Ptr+str_length,max_length-str_length,fill_char);
+    memset(Ptr+str_length, fill_char, max_length-str_length);
     str_length=max_length;
   }
   return FALSE;
@@ -419,6 +435,9 @@ bool String::append(const String &s)
 {
   if (s.length())
   {
+    DBUG_ASSERT(!this->uses_buffer_owned_by(&s));
+    DBUG_ASSERT(!s.uses_buffer_owned_by(this));
+
     if (realloc(str_length+s.length()))
       return TRUE;
     memcpy(Ptr+str_length,s.ptr(),s.length());
@@ -488,7 +507,7 @@ bool String::append_ulonglong(ulonglong val)
   with character set recoding
 */
 
-bool String::append(const char *s,uint32 arg_length, CHARSET_INFO *cs)
+bool String::append(const char *s,uint32 arg_length, const CHARSET_INFO *cs)
 {
   uint32 offset;
   
@@ -502,7 +521,7 @@ bool String::append(const char *s,uint32 arg_length, CHARSET_INFO *cs)
       add_length= arg_length + offset;
       if (realloc(str_length + add_length))
         return TRUE;
-      bzero((char*) Ptr + str_length, offset);
+      memset(Ptr + str_length, 0, offset);
       memcpy(Ptr + str_length + offset, s, arg_length);
       str_length+= add_length;
       return FALSE;
@@ -538,6 +557,24 @@ bool String::append(IO_CACHE* file, uint32 arg_length)
   return FALSE;
 }
 
+
+/**
+  Append a parenthesized number to String.
+  Used in various pieces of SHOW related code.
+
+  @param nr     Number
+  @param radix  Radix, optional parameter, 10 by default.
+*/
+bool String::append_parenthesized(long nr, int radix)
+{
+  char buff[64], *end;
+  buff[0]= '(';
+  end= int10_to_str(nr, buff + 1, radix);
+  *end++ = ')';
+  return append(buff, (uint) (end - buff));
+}
+
+
 bool String::append_with_prefill(const char *s,uint32 arg_length,
 		 uint32 full_length, char fill_char)
 {
@@ -548,57 +585,14 @@ bool String::append_with_prefill(const char *s,uint32 arg_length,
   t_length= full_length - arg_length;
   if (t_length > 0)
   {
-    bfill(Ptr+str_length, t_length, fill_char);
+    memset(Ptr+str_length, fill_char, t_length);
     str_length=str_length + t_length;
   }
   append(s, arg_length);
   return FALSE;
 }
 
-bool String::append_identifier(const char *name,
-			       uint length,
-			       CHARSET_INFO *ci,
-			       int quote_char)
-{
-  const char *name_end;
-  char q= (char)quote_char;
-  CHARSET_INFO *lci = ci ? ci : charset();
-
-  if (quote_char == EOF)
-    return append(name, length, charset());
-
-  /*
-    The identifier must be quoted as it includes a quote character or
-   it's a keyword
-  */
-
-  (void)reserve(length*2 + 2);
-  if (append(&q, 1, lci))
-    return true;
-
-  for (name_end= name+length ; name < name_end ; name+= length)
-  {
-    uchar chr= (uchar) *name;
-    length= my_mbcharlen(lci, chr);
-    /*
-      my_mbcharlen can return 0 on a wrong multibyte
-      sequence. It is possible when upgrading from 4.0,
-      and identifier contains some accented characters.
-      The manual says it does not work. So we'll just
-      change length to 1 not to hang in the endless loop.
-    */
-    if (!length)
-      length= 1;
-    if (length == 1 && chr == (uchar) q &&
-        append(&q, 1, lci))
-      return true;
-    if (append(name, length, lci))
-      return true;
-  }
-  return append(&q, 1, lci);
-}
-
-uint32 String::numchars()
+uint32 String::numchars() const
 {
   return str_charset->cset->numchars(str_charset, Ptr, Ptr+str_length);
 }
@@ -773,7 +767,7 @@ void String::qs_append(uint i)
 */
 
 
-int sortcmp(const String *s,const String *t, CHARSET_INFO *cs)
+int sortcmp(const String *s,const String *t, const CHARSET_INFO *cs)
 {
  return cs->coll->strnncollsp(cs,
                               (uchar *) s->ptr(),s->length(),
@@ -806,18 +800,42 @@ int stringcmp(const String *s,const String *t)
   return (cmp) ? cmp : (int) (s_len - t_len);
 }
 
+/**
+  Makes a copy of a String's buffer unless it's already heap-allocated.
 
+  If the buffer ('str') of 'from' is on the heap, this function returns
+  'from', possibly re-allocated to be at least from_length bytes long.
+  It is also the case if from==to or to==NULL.
+  Otherwise, this function makes and returns a copy of "from" into "to"; the
+  buffer of "to" is heap-allocated; a pre-condition is that from->str and
+  to->str must point to non-overlapping buffers.
+  The logic behind this complex design, is that a caller, typically a
+  val_str() function, sometimes has an input String ('from') which buffer it
+  wants to modify; but this String's buffer may or not be heap-allocated; if
+  it's not heap-allocated it is possibly in static storage or belongs to an
+  outer context, and thus should not be modified; in that case the caller
+  wants a heap-allocated copy which it can freely modify.
+
+  @param  to    destination string
+  @param  from  source string
+  @param  from_length  destination string will hold at least from_length bytes.
+*/
 String *copy_if_not_alloced(String *to,String *from,uint32 from_length)
 {
-  if (from->Alloced_length >= from_length)
+  if (from->alloced && from->Alloced_length >= from_length)
     return from;
   if ((from->alloced && (from->Alloced_length != 0)) || !to || from == to)
   {
-    (void) from->realloc(from_length);
+    (void) from->realloc(from_length, true);
     return from;
   }
-  if (to->realloc(from_length))
+  if (to->realloc(from_length, true))
     return from;				// Actually an error
+
+  // from and to should not be overlapping
+  DBUG_ASSERT(!to->uses_buffer_owned_by(from));
+  DBUG_ASSERT(!from->uses_buffer_owned_by(to));
+
   if ((to->str_length=min(from->str_length,from_length)))
     memcpy(to->Ptr,from->Ptr,to->str_length);
   to->str_charset=from->str_charset;
@@ -829,140 +847,6 @@ String *copy_if_not_alloced(String *to,String *from,uint32 from_length)
   Help functions
 ****************************************************************************/
 
-/*
-  copy a string from one character set to another
-  
-  SYNOPSIS
-    copy_and_convert()
-    to			Store result here
-    to_cs		Character set of result string
-    from		Copy from here
-    from_length		Length of from string
-    from_cs		From character set
-
-  NOTES
-    'to' must be big enough as form_length * to_cs->mbmaxlen
-
-  RETURN
-    length of bytes copied to 'to'
-*/
-
-
-static uint32
-copy_and_convert_extended(char *to, uint32 to_length, CHARSET_INFO *to_cs, 
-                          const char *from, uint32 from_length,
-                          CHARSET_INFO *from_cs,
-                          uint *errors)
-{
-  int         cnvres;
-  my_wc_t     wc;
-  const uchar *from_end= (const uchar*) from+from_length;
-  char *to_start= to;
-  uchar *to_end= (uchar*) to+to_length;
-  my_charset_conv_mb_wc mb_wc= from_cs->cset->mb_wc;
-  my_charset_conv_wc_mb wc_mb= to_cs->cset->wc_mb;
-  uint error_count= 0;
-
-  while (1)
-  {
-    if ((cnvres= (*mb_wc)(from_cs, &wc, (uchar*) from,
-				      from_end)) > 0)
-      from+= cnvres;
-    else if (cnvres == MY_CS_ILSEQ)
-    {
-      error_count++;
-      from++;
-      wc= '?';
-    }
-    else if (cnvres > MY_CS_TOOSMALL)
-    {
-      /*
-        A correct multibyte sequence detected
-        But it doesn't have Unicode mapping.
-      */
-      error_count++;
-      from+= (-cnvres);
-      wc= '?';
-    }
-    else
-      break;  // Not enough characters
-
-outp:
-    if ((cnvres= (*wc_mb)(to_cs, wc, (uchar*) to, to_end)) > 0)
-      to+= cnvres;
-    else if (cnvres == MY_CS_ILUNI && wc != '?')
-    {
-      error_count++;
-      wc= '?';
-      goto outp;
-    }
-    else
-      break;
-  }
-  *errors= error_count;
-  return (uint32) (to - to_start);
-}
-
-
-/*
-  Optimized for quick copying of ASCII characters in the range 0x00..0x7F.
-*/
-uint32
-copy_and_convert(char *to, uint32 to_length, CHARSET_INFO *to_cs, 
-                 const char *from, uint32 from_length, CHARSET_INFO *from_cs,
-                 uint *errors)
-{
-  /*
-    If any of the character sets is not ASCII compatible,
-    immediately switch to slow mb_wc->wc_mb method.
-  */
-  if ((to_cs->state | from_cs->state) & MY_CS_NONASCII)
-    return copy_and_convert_extended(to, to_length, to_cs,
-                                     from, from_length, from_cs, errors);
-
-  uint32 length= min(to_length, from_length), length2= length;
-
-#if defined(__i386__)
-  /*
-    Special loop for i386, it allows to refer to a
-    non-aligned memory block as UINT32, which makes
-    it possible to copy four bytes at once. This
-    gives about 10% performance improvement comparing
-    to byte-by-byte loop.
-  */
-  for ( ; length >= 4; length-= 4, from+= 4, to+= 4)
-  {
-    if ((*(uint32*)from) & 0x80808080)
-      break;
-    *((uint32*) to)= *((const uint32*) from);
-  }
-#endif
-
-  for (; ; *to++= *from++, length--)
-  {
-    if (!length)
-    {
-      *errors= 0;
-      return length2;
-    }
-    if (*((unsigned char*) from) > 0x7F) /* A non-ASCII character */
-    {
-      uint32 copied_length= length2 - length;
-      to_length-= copied_length;
-      from_length-= copied_length;
-      return copied_length + copy_and_convert_extended(to, to_length,
-                                                       to_cs,
-                                                       from, from_length,
-                                                       from_cs,
-                                                       errors);
-    }
-  }
-
-  DBUG_ASSERT(FALSE); // Should never get to here
-  return 0;           // Make compiler happy
-}
-
-
 /**
   Copy string with HEX-encoding of "bad" characters.
 
@@ -984,7 +868,7 @@ copy_and_convert(char *to, uint32 to_length, CHARSET_INFO *to_cs,
 */
 
 size_t
-my_copy_with_hex_escaping(CHARSET_INFO *cs,
+my_copy_with_hex_escaping(const CHARSET_INFO *cs,
                           char *dst, size_t dstlen,
                           const char *src, size_t srclen)
 {
@@ -1053,9 +937,9 @@ my_copy_with_hex_escaping(CHARSET_INFO *cs,
 
 
 uint32
-well_formed_copy_nchars(CHARSET_INFO *to_cs,
+well_formed_copy_nchars(const CHARSET_INFO *to_cs,
                         char *to, uint to_length,
-                        CHARSET_INFO *from_cs,
+                        const CHARSET_INFO *from_cs,
                         const char *from, uint from_length,
                         uint nchars,
                         const char **well_formed_error_pos,
@@ -1099,7 +983,7 @@ well_formed_copy_nchars(CHARSET_INFO *to_cs,
           0x01 -> 0x0001
         */
         uint pad_length= to_cs->mbminlen - from_offset;
-        bzero(to, pad_length);
+        memset(to, 0, pad_length);
         memmove(to + pad_length, from, from_offset);
         /*
           In some cases left zero-padding can create an incorrect character.
@@ -1199,47 +1083,39 @@ outp:
 
 
 
-/*
-  Append characters to a single-quoted string '...', escaping special
-  characters as necessary.
-  Does not add the enclosing quotes, this is left up to caller.
-*/
-void String::append_for_single_quote(const char *st, uint len)
+
+void String::print(String *str)
 {
-  const char *end= st+len;
+  char *st= (char*)Ptr, *end= st+str_length;
   for (; st < end; st++)
   {
     uchar c= *st;
     switch (c)
     {
     case '\\':
-      append(STRING_WITH_LEN("\\\\"));
+      str->append(STRING_WITH_LEN("\\\\"));
       break;
     case '\0':
-      append(STRING_WITH_LEN("\\0"));
+      str->append(STRING_WITH_LEN("\\0"));
       break;
     case '\'':
-      append(STRING_WITH_LEN("\\'"));
+      str->append(STRING_WITH_LEN("\\'"));
       break;
     case '\n':
-      append(STRING_WITH_LEN("\\n"));
+      str->append(STRING_WITH_LEN("\\n"));
       break;
     case '\r':
-      append(STRING_WITH_LEN("\\r"));
+      str->append(STRING_WITH_LEN("\\r"));
       break;
     case '\032': // Ctrl-Z
-      append(STRING_WITH_LEN("\\Z"));
+      str->append(STRING_WITH_LEN("\\Z"));
       break;
     default:
-      append(c);
+      str->append(c);
     }
   }
 }
 
-void String::print(String *str)
-{
-  str->append_for_single_quote(Ptr, str_length);
-}
 
 /*
   Exchange state of this object and argument.
@@ -1257,7 +1133,7 @@ void String::swap(String &s)
   swap_variables(uint32, str_length, s.str_length);
   swap_variables(uint32, Alloced_length, s.Alloced_length);
   swap_variables(bool, alloced, s.alloced);
-  swap_variables(CHARSET_INFO*, str_charset, s.str_charset);
+  swap_variables(const CHARSET_INFO *, str_charset, s.str_charset);
 }
 
 
@@ -1283,7 +1159,7 @@ void String::swap(String &s)
 
 uint convert_to_printable(char *to, size_t to_len,
                           const char *from, size_t from_len,
-                          CHARSET_INFO *from_cs, size_t nbytes /*= 0*/)
+                          const CHARSET_INFO *from_cs, size_t nbytes /*= 0*/)
 {
   /* needs at least 8 bytes for '\xXX...' and zero byte */
   DBUG_ASSERT(to_len >= 8);
@@ -1362,7 +1238,7 @@ uint convert_to_printable(char *to, size_t to_len,
                                   // encoding for some input character
         return true
 */
-bool validate_string(CHARSET_INFO *cs, const char *str, uint32 length,
+bool validate_string(const CHARSET_INFO *cs, const char *str, uint32 length,
                      size_t *valid_length, bool *length_error)
 {
   if (cs->mbmaxlen > 1)
