@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -502,6 +502,7 @@ int mysql_update(THD *thd,
       goto exit_without_my_ok;
     }
   }
+
   if (select_lex->has_ft_funcs() && init_ftfuncs(thd, select_lex, 1))
     goto exit_without_my_ok;
 
@@ -740,6 +741,13 @@ int mysql_update(THD *thd,
       check_constant_expressions(values))
     read_removal= table->check_read_removal(select->quick->index);
 
+  error= table->file->ha_fast_update(thd, fields, values, conds);
+  if (error == 0)
+    error= -1; // error < 0 means really no error at all (see below)
+  else if (error != ENOTSUP) {
+    error= 1;
+  }
+  else
   while (!(error=info.read_record(&info)) && !thd->killed)
   {
     thd->inc_examined_row_count(1);
