@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_6_SOURCE_TARBALLS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED
-# define YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_6_SOURCE_TARBALLS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_6_SOURCE_TARBALLS_SQL_SQL_YACC_H_INCLUDED
+# define YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_6_SOURCE_TARBALLS_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -1330,7 +1330,7 @@ extern int MYSQLdebug;
 
 union YYSTYPE
 {
-#line 968 "/var/lib/jenkins/jobs/percona-server-5.6-source-tarballs/workspace/sql/sql_yacc.yy" /* yacc.c:1909  */
+#line 968 "/mnt/workspace/percona-server-5.6-source-tarballs/sql/sql_yacc.yy" /* yacc.c:1909  */
 
   int  num;
   ulong ulong_num;
@@ -1389,7 +1389,7 @@ union YYSTYPE
   List<Condition_information_item> *cond_info_list;
   bool is_not_empty;
 
-#line 1393 "/var/lib/jenkins/jobs/percona-server-5.6-source-tarballs/workspace/sql/sql_yacc.h" /* yacc.c:1909  */
+#line 1393 "/mnt/workspace/percona-server-5.6-source-tarballs/sql/sql_yacc.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
@@ -1401,4 +1401,4 @@ typedef union YYSTYPE YYSTYPE;
 
 int MYSQLparse (class THD *YYTHD);
 
-#endif /* !YY_MYSQL_VAR_LIB_JENKINS_JOBS_PERCONA_SERVER_5_6_SOURCE_TARBALLS_WORKSPACE_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MYSQL_MNT_WORKSPACE_PERCONA_SERVER_5_6_SOURCE_TARBALLS_SQL_SQL_YACC_H_INCLUDED  */
