@@ -20,11 +20,18 @@
 #include "probes_mysql.h"
 #include "ha_blackhole.h"
 #include "sql_class.h"                          // THD, SYSTEM_THREAD_SLAVE_*
+#include "rpl_rli.h" // THD::rli_slave::rows_query_ev
 
-static bool is_slave_applier(THD *thd)
+static inline bool is_slave_applier(const THD &thd)
 {
-  return thd->system_thread == SYSTEM_THREAD_SLAVE_SQL ||
-    thd->system_thread == SYSTEM_THREAD_SLAVE_WORKER;
+  return thd.system_thread == SYSTEM_THREAD_SLAVE_SQL ||
+    thd.system_thread == SYSTEM_THREAD_SLAVE_WORKER;
+}
+
+static inline bool pretend_for_slave(const THD &thd)
+{
+  return is_slave_applier(thd) &&
+    (thd.rli_slave->rows_query_ev || thd.query() == NULL);
 }
 
 /* Static declarations for handlerton */
@@ -120,7 +127,7 @@ int ha_blackhole::update_row(const uchar *old_data, uchar *new_data)
 {
   DBUG_ENTER("ha_blackhole::update_row");
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     DBUG_RETURN(0);
   DBUG_RETURN(HA_ERR_WRONG_COMMAND);
 }
@@ -129,7 +136,7 @@ int ha_blackhole::delete_row(const uchar *buf)
 {
   DBUG_ENTER("ha_blackhole::delete_row");
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     DBUG_RETURN(0);
   DBUG_RETURN(HA_ERR_WRONG_COMMAND);
 }
@@ -148,7 +155,7 @@ int ha_blackhole::rnd_next(uchar *buf)
   MYSQL_READ_ROW_START(table_share->db.str, table_share->table_name.str,
                        TRUE);
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     rc= 0;
   else
     rc= HA_ERR_END_OF_FILE;
@@ -181,7 +188,7 @@ int ha_blackhole::info(uint flag)
 {
   DBUG_ENTER("ha_blackhole::info");
 
-  memset(&stats, 0, sizeof(stats));
+  memset(static_cast<void*>(&stats), 0, sizeof(stats));
   if (flag & HA_STATUS_AUTO)
     stats.auto_increment_value= 1;
   DBUG_RETURN(0);
@@ -239,7 +246,7 @@ int ha_blackhole::index_read_map(uchar * buf, const uchar * key,
   DBUG_ENTER("ha_blackhole::index_read");
   MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     rc= 0;
   else
     rc= HA_ERR_END_OF_FILE;
@@ -257,7 +264,7 @@ int ha_blackhole::index_read_idx_map(uchar * buf, uint idx, const uchar * key,
   DBUG_ENTER("ha_blackhole::index_read_idx");
   MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     rc= 0;
   else
     rc= HA_ERR_END_OF_FILE;
@@ -274,7 +281,7 @@ int ha_blackhole::index_read_last_map(uchar * buf, const uchar * key,
   DBUG_ENTER("ha_blackhole::index_read_last");
   MYSQL_INDEX_READ_ROW_START(table_share->db.str, table_share->table_name.str);
   THD *thd= ha_thd();
-  if (is_slave_applier(thd) && thd->query() == NULL)
+  if (pretend_for_slave(*thd))
     rc= 0;
   else
     rc= HA_ERR_END_OF_FILE;
