@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +15,6 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #define MYSQL_SERVER 1
 #include "sql_priv.h"
 #include "probes_mysql.h"
@@ -101,7 +97,7 @@ const char **ha_heap::bas_ext() const
 
 int ha_heap::open(const char *name, int mode, uint test_if_locked)
 {
-  internal_table= test(test_if_locked & HA_OPEN_INTERNAL_TABLE);
+  internal_table= MY_TEST(test_if_locked & HA_OPEN_INTERNAL_TABLE);
   if (internal_table || (!(file= heap_open(name, mode)) && my_errno == ENOENT))
   {
     HP_CREATE_INFO create_info;
@@ -118,7 +114,7 @@ int ha_heap::open(const char *name, int mode, uint test_if_locked)
     if (rc)
       goto end;
 
-    implicit_emptied= test(created_new_share);
+    implicit_emptied= MY_TEST(created_new_share);
     if (internal_table)
       file= heap_open_from_share(internal_share, mode);
     else
@@ -216,14 +212,14 @@ void ha_heap::update_key_stats()
     if (key->algorithm != HA_KEY_ALG_BTREE)
     {
       if (key->flags & HA_NOSAME)
-        key->rec_per_key[key->key_parts-1]= 1;
+        key->rec_per_key[key->user_defined_key_parts - 1]= 1;
       else
       {
         ha_rows hash_buckets= file->s->keydef[i].hash_buckets;
         uint no_records= hash_buckets ? (uint) (file->s->records/hash_buckets) : 2;
         if (no_records < 2)
           no_records= 2;
-        key->rec_per_key[key->key_parts-1]= no_records;
+        key->rec_per_key[key->user_defined_key_parts - 1]= no_records;
       }
     }
   }
@@ -237,8 +233,6 @@ int ha_heap::write_row(uchar * buf)
 {
   int res;
   ha_statistic_increment(&SSV::ha_write_count);
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
-    table->timestamp_field->set_time();
   if (table->next_number_field && buf == table->record[0])
   {
     if ((res= update_auto_increment()))
@@ -261,8 +255,6 @@ int ha_heap::update_row(const uchar * old_data, uchar * new_data)
 {
   int res;
   ha_statistic_increment(&SSV::ha_update_count);
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_UPDATE)
-    table->timestamp_field->set_time();
   res= heap_update(file,old_data,new_data);
   if (!res && ++records_changed*HEAP_STATS_UPDATE_THRESHOLD > 
               file->s->records)
@@ -652,7 +644,7 @@ ha_rows ha_heap::records_in_range(uint inx, key_range *min_key,
 
   /* Assert that info() did run. We need current statistics here. */
   DBUG_ASSERT(key_stat_version == file->s->key_stat_version);
-  return key->rec_per_key[key->key_parts-1];
+  return key->rec_per_key[key->user_defined_key_parts - 1];
 }
 
 
@@ -671,7 +663,7 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
   bool found_real_auto_increment= 0;
   uint blobs= 0;
 
-  bzero(hp_create_info, sizeof(*hp_create_info));
+  memset(hp_create_info, 0, sizeof(*hp_create_info));
 
   if (!(columndef= (HP_COLUMNDEF*) my_malloc(column_count *
                                              sizeof(HP_COLUMNDEF),
@@ -689,8 +681,7 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
     if (field->null_bit)
     {
       column->null_bit= field->null_bit;
-      column->null_pos= (uint) (field->null_ptr -
-                                (uchar*) table_arg->record[0]);
+      column->null_pos= field->null_offset();
     }
     else
     {
@@ -715,7 +706,7 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
   }
 
   for (key= parts= 0; key < keys; key++)
-    parts+= table_arg->key_info[key].key_parts;
+    parts+= table_arg->key_info[key].user_defined_key_parts;
 
   if (!(keydef= (HP_KEYDEF*) my_malloc(keys * sizeof(HP_KEYDEF) +
 				       parts * sizeof(HA_KEYSEG),
@@ -729,9 +720,9 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
   {
     KEY *pos= table_arg->key_info+key;
     KEY_PART_INFO *key_part=     pos->key_part;
-    KEY_PART_INFO *key_part_end= key_part + pos->key_parts;
+    KEY_PART_INFO *key_part_end= key_part + pos->user_defined_key_parts;
 
-    keydef[key].keysegs=   (uint) pos->key_parts;
+    keydef[key].keysegs=   (uint) pos->user_defined_key_parts;
     keydef[key].flag=      (pos->flags & (HA_NOSAME | HA_NULL_ARE_EQUAL));
     keydef[key].seg=       seg;
 
@@ -782,10 +773,10 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
         seg->charset= &my_charset_bin;
       else
         seg->charset= field->charset_for_protocol();
-      if (field->null_ptr)
+      if (field->real_maybe_null())
       {
 	seg->null_bit= field->null_bit;
-	seg->null_pos= (uint) (field->null_ptr - (uchar*) table_arg->record[0]);
+	seg->null_pos= field->null_offset();
       }
       else
       {
@@ -826,7 +817,7 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
         keydef[key].flag|= HA_VAR_LENGTH_KEY;
         /* Save number of bytes used to store length */
         if (seg->flag & HA_BLOB_PART)
-          seg->bit_start= field->pack_length() - share->blob_ptr_size;
+          seg->bit_start= field->pack_length() - portable_sizeof_char_ptr;
         else
           seg->bit_start= 1;
         break;
@@ -837,7 +828,7 @@ heap_prepare_hp_create_info(TABLE *table_arg, bool internal_table,
         keydef[key].flag|= HA_VAR_LENGTH_KEY;
         /* Save number of bytes used to store length */
         if (seg->flag & HA_BLOB_PART)
-          seg->bit_start= field->pack_length() - share->blob_ptr_size;
+          seg->bit_start= field->pack_length() - portable_sizeof_char_ptr;
         else
           seg->bit_start= 2;
         /*
