@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -263,7 +263,7 @@ ulong hp_hashnr(HP_KEYDEF *keydef, const uchar *key)
     }
     if (seg->type == HA_KEYTYPE_TEXT)
     {
-       CHARSET_INFO *cs= seg->charset;
+       const CHARSET_INFO *cs= seg->charset;
        uint length= seg->length;
        if (cs->mbmaxlen > 1)
        {
@@ -275,7 +275,7 @@ ulong hp_hashnr(HP_KEYDEF *keydef, const uchar *key)
     }
     else if (seg->type == HA_KEYTYPE_VARTEXT1)  /* Any VARCHAR segments */
     {
-       CHARSET_INFO *cs= seg->charset;
+       const CHARSET_INFO *cs= seg->charset;
        uint pack_length= 2;                     /* Key packing is constant */
        uint length= uint2korr(pos);
        if (cs->mbmaxlen > 1)
@@ -322,7 +322,7 @@ ulong hp_rec_hashnr(HP_KEYDEF *keydef, const uchar *rec)
     }
     if (seg->type == HA_KEYTYPE_TEXT)
     {
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       uint char_length= seg->length;
       if (cs->mbmaxlen > 1)
       {
@@ -334,7 +334,7 @@ ulong hp_rec_hashnr(HP_KEYDEF *keydef, const uchar *rec)
     }
     else if (seg->type == HA_KEYTYPE_VARTEXT1)  /* Any VARCHAR segments */
     {
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       uint pack_length= seg->bit_start;
       uint length= hp_calc_blob_length(pack_length, pos);
 
@@ -526,7 +526,7 @@ int hp_rec_key_cmp(HP_KEYDEF *keydef, const uchar *rec1, const uchar *rec2,
     }
     if (seg->type == HA_KEYTYPE_TEXT)
     {
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       uint char_length1;
       uint char_length2;
       uchar *pos1= (uchar*)rec1 + seg->start;
@@ -554,8 +554,7 @@ int hp_rec_key_cmp(HP_KEYDEF *keydef, const uchar *rec1, const uchar *rec2,
       uchar *pos2= (uchar*) rec2 + seg->start;
       uint char_length1, char_length2;
       uint pack_length= seg->bit_start;
-      CHARSET_INFO *cs= seg->charset;
-
+      const CHARSET_INFO *cs= seg->charset;
       char_length1= hp_calc_blob_length(pack_length, pos1);
       char_length2= hp_calc_blob_length(pack_length, pos2);
       pos1+= pack_length;
@@ -606,7 +605,7 @@ int hp_key_cmp(HP_KEYDEF *keydef, const uchar *rec, const uchar *key)
   {
     if (seg->null_bit)
     {
-      int found_null=test(rec[seg->null_pos] & seg->null_bit);
+      int found_null=MY_TEST(rec[seg->null_pos] & seg->null_bit);
       if (found_null != (int) *key++)
 	return 1;
       if (found_null)
@@ -619,7 +618,7 @@ int hp_key_cmp(HP_KEYDEF *keydef, const uchar *rec, const uchar *key)
     }
     if (seg->type == HA_KEYTYPE_TEXT)
     {
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       uint char_length_key;
       uint char_length_rec;
       uchar *pos= (uchar*) rec + seg->start;
@@ -645,7 +644,7 @@ int hp_key_cmp(HP_KEYDEF *keydef, const uchar *rec, const uchar *key)
     else if (seg->type == HA_KEYTYPE_VARTEXT1)  /* Any VARCHAR segments */
     {
       uchar *pos= (uchar*) rec + seg->start;
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       uint pack_length= seg->bit_start;
       uint char_length_rec= (pack_length == 1 ? (uint) *(uchar*) pos :
                              uint2korr(pos));
@@ -662,6 +661,10 @@ int hp_key_cmp(HP_KEYDEF *keydef, const uchar *rec, const uchar *key)
         char_length2= my_charpos(cs, pos, pos + char_length_rec, char_length2);
         set_if_smaller(char_length_rec, char_length2);
       }
+      else
+      {
+        set_if_smaller(char_length_rec, seg->length);
+      }
 
       if (cs->coll->strnncollsp(seg->charset,
                                 (uchar*) pos, char_length_rec,
@@ -714,16 +717,16 @@ void hp_make_key(HP_KEYDEF *keydef, uchar *key, const uchar *rec)
 
   for (seg=keydef->seg,endseg=seg+keydef->keysegs ; seg < endseg ; seg++)
   {
-    CHARSET_INFO *cs= seg->charset;
+    const CHARSET_INFO *cs= seg->charset;
     uint char_length= seg->length;
     const uchar *pos= rec + seg->start;
     if (seg->null_bit)
-      *key++= test(rec[seg->null_pos] & seg->null_bit);
+      *key++= MY_TEST(rec[seg->null_pos] & seg->null_bit);
 
     if (seg->flag & HA_BLOB_PART)
     {
       uint tmp_length= hp_calc_blob_length(seg->bit_start, pos);
-      uint length= min(seg->length, tmp_length);
+      uint length= MY_MIN(seg->length, tmp_length);
 
       memcpy(&pos, rec + seg->bit_start, sizeof(char *));
       if (cs->mbmaxlen > 1)
@@ -770,7 +773,7 @@ uint hp_rb_make_key(HP_KEYDEF *keydef, uchar *key,
     uint char_length;
     if (seg->null_bit)
     {
-      if (!(*key++= 1 - test(rec[seg->null_pos] & seg->null_bit)))
+      if (!(*key++= 1 - MY_TEST(rec[seg->null_pos] & seg->null_bit)))
         continue;
     }
     if (seg->flag & HA_SWAP_KEY)
@@ -783,10 +786,10 @@ uint hp_rb_make_key(HP_KEYDEF *keydef, uchar *key,
       {
 	float nr;
 	float4get(nr, pos);
-	if (isnan(nr))
+	if (my_isnan(nr))
 	{
 	  /* Replace NAN with zero */
- 	  bzero(key, length);
+ 	  memset(key, 0, length);
 	  key+= length;
 	  continue;
 	}
@@ -795,9 +798,9 @@ uint hp_rb_make_key(HP_KEYDEF *keydef, uchar *key,
       {
 	double nr;
 	float8get(nr, pos);
-	if (isnan(nr))
+	if (my_isnan(nr))
 	{
- 	  bzero(key, length);
+ 	  memset(key, 0, length);
 	  key+= length;
 	  continue;
 	}
@@ -811,14 +814,14 @@ uint hp_rb_make_key(HP_KEYDEF *keydef, uchar *key,
       continue;
     }
 
-    if (seg->flag & HA_VAR_LENGTH_PART)
+    if (seg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART))
     {
       uchar *pos=      (uchar*) rec + seg->start;
       uint length=     seg->length;
       uint pack_length= seg->bit_start;
       uint tmp_length= (pack_length == 1 ? (uint) *(uchar*) pos :
                         uint2korr(pos));
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       char_length= length/cs->mbmaxlen;
 
       pos+= pack_length;			/* Skip VARCHAR length */
@@ -833,8 +836,8 @@ uint hp_rb_make_key(HP_KEYDEF *keydef, uchar *key,
     {
       uchar *pos=      (uchar*) rec + seg->start;
       uint tmp_length= hp_calc_blob_length(seg->bit_start, pos);
-      uint length= min(seg->length, tmp_length);
-      CHARSET_INFO *cs= seg->charset;
+      uint length= MY_MIN(seg->length, tmp_length);
+      const CHARSET_INFO *cs= seg->charset;
       char_length= seg->length / cs->mbmaxlen;
 
       /* check_one_rb_key() calls hp_rb_make_key() for already packed records */
@@ -885,9 +888,19 @@ uint hp_rb_pack_key(HP_KEYDEF *keydef, uchar *key, const uchar *old,
     keypart_map>>= 1;
     if (seg->null_bit)
     {
+      /* Convert NULL from MySQL representation into HEAP's. */
       if (!(*key++= (char) 1 - *old++))
+      {
+        /*
+          Skip length part of a variable length field.
+          Length of key-part used with heap_rkey() always 2.
+          See also hp_hashnr().
+        */
+        if (seg->flag & (HA_VAR_LENGTH_PART | HA_BLOB_PART))
+          old+= 2;
         continue;
       }
+    }
     if (seg->flag & HA_SWAP_KEY)
     {
       uint length= seg->length;
@@ -904,7 +917,7 @@ uint hp_rb_pack_key(HP_KEYDEF *keydef, uchar *key, const uchar *old,
       /* Length of key-part used with heap_rkey() always 2 */
       uint tmp_length=uint2korr(old);
       uint length= seg->length;
-      CHARSET_INFO *cs= seg->charset;
+      const CHARSET_INFO *cs= seg->charset;
       char_length= length/cs->mbmaxlen;
 
       old+= 2;
@@ -933,7 +946,7 @@ uint hp_rb_pack_key(HP_KEYDEF *keydef, uchar *key, const uchar *old,
 
 
 uint hp_rb_key_length(HP_KEYDEF *keydef, 
-		      const uchar *key __attribute__((unused)))
+		      const uchar *key MY_ATTRIBUTE((unused)))
 {
   return keydef->length;
 }
