@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
    reserved
 
    This program is free software; you can redistribute it and/or modify
@@ -66,7 +66,7 @@ int main(int argc, char *argv[])
   file=file2=0;
   get_options(argc,argv);
 
-  bzero(&hp_create_info, sizeof(hp_create_info));
+  memset(&hp_create_info, 0, sizeof(hp_create_info));
   hp_create_info.max_table_size= 1024L*1024L*1024L;
   hp_create_info.keys= keys;
   hp_create_info.keydef= keyinfo;
@@ -141,8 +141,8 @@ int main(int argc, char *argv[])
   columndef[3].null_bit= 1;
   columndef[3].null_pos= 38;
 
-  bzero((char*) key1,sizeof(key1));
-  bzero((char*) key3,sizeof(key3));
+  memset(key1, 0, sizeof(key1));
+  memset(key3, 0, sizeof(key3));
 
   printf("- Creating heap-file\n");
   if (heap_create(filename, &hp_create_info,
@@ -156,7 +156,9 @@ int main(int argc, char *argv[])
 
   for (i=0 ; i < recant ; i++)
   {
-    n1=rnd(1000); n2=rnd(100); n3=rnd(min(recant*5,MAX_RECORDS));
+    n1= rnd(1000);
+    n2= rnd(100);
+    n3= rnd(MY_MIN(recant * 5, MAX_RECORDS));
     make_record(record,n1,n2,n3,"Pos",write_count);
 
     if (heap_write(file,record))
@@ -232,7 +234,9 @@ int main(int argc, char *argv[])
   printf("- Update\n");
   for (i=0 ; i < write_count/10 ; i++)
   {
-    n1=rnd(1000); n2=rnd(100); n3=rnd(min(recant*2,MAX_RECORDS));
+    n1= rnd(1000);
+    n2= rnd(100);
+    n3= rnd(MY_MIN(recant * 2, MAX_RECORDS));
     make_record(record2, n1, n2, n3, "XXX", update);
     if (rnd(2) == 1)
     {
@@ -410,7 +414,7 @@ int main(int argc, char *argv[])
   {
     if (!error)
       pos--;
-    if (i-- == 0)
+    if (!error && (i-- == 0))
     {
       bmove(record3,record,reclength);
       position=heap_position(file);
@@ -681,7 +685,7 @@ static int rnd(int max_value)
 } /* rnd */
 
 
-static sig_handler endprog(int sig_number __attribute__((unused)))
+static sig_handler endprog(int sig_number MY_ATTRIBUTE((unused)))
 {
   {
     hp_panic(HA_PANIC_CLOSE);
@@ -701,7 +705,7 @@ static int calc_check(uchar *buf, uint length)
 static void make_record(uchar *record, uint n1, uint n2, uint n3,
 			const char *mark, uint count)
 {
-  bfill(record,reclength,' ');
+  memset(record, ' ', reclength);
   sprintf((char*) record,"%6d:%4d:%8d:%3.3s: %4d",
 	  n1,n2,n3,mark,count);
   record[37]='A';				/* Store A in null key */
