@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -229,10 +229,6 @@ int mi_lock_database(MI_INFO *info, int lock_type)
       info->invalidator=info->s->invalidator;
       share->w_locks++;
       share->tot_locks++;
-
-      DBUG_EXECUTE_IF("simulate_incorrect_share_wlock_value",
-                      DEBUG_SYNC_C("after_share_wlock_increment"););
-
       info->s->in_use= list_add(info->s->in_use, &info->in_use);
       break;
     default:
@@ -456,8 +452,8 @@ int _mi_writeinfo(MI_INFO *info, uint operation)
 #ifdef _WIN32
       if (myisam_flush)
       {
-        if (share->file_map)
-          my_msync(info->dfile, share->file_map, share->mmaped_length, MS_SYNC);
+       if (share->file_map)
+         my_msync(info->dfile, share->file_map, share->mmaped_length, MS_SYNC);
         mysql_file_sync(share->kfile, 0);
         mysql_file_sync(info->dfile, 0);
       }
@@ -575,5 +571,5 @@ int _mi_decrement_open_count(MI_INFO *info)
     if (!lock_error)
       lock_error=mi_lock_database(info,old_lock);
   }
-  return test(lock_error || write_error);
+  return MY_TEST(lock_error || write_error);
 }
