@@ -98,7 +98,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
   lock_error=1;
   errpos=0;
   head_length=sizeof(share_buff.state.header);
-  bzero((uchar*) &info,sizeof(info));
+  memset(&info, 0, sizeof(info));
 
   realpath_err= my_realpath(name_buff,
                   fn_format(org_name,name,"",MI_NAME_IEXT,4),MYF(0));
@@ -122,7 +122,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
   if (!old_info)
   {
     share= &share_buff;
-    bzero((uchar*) &share_buff,sizeof(share_buff));
+    memset(&share_buff, 0, sizeof(share_buff));
     share_buff.state.rec_per_key_part=rec_per_key_part;
     share_buff.state.key_root=key_root;
     share_buff.state.key_del=key_del;
@@ -338,7 +338,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
     strmov(share->index_file_name,  index_name);
     strmov(share->data_file_name,   data_name);
 
-    share->blocksize=min(IO_SIZE,myisam_block_size);
+    share->blocksize= MY_MIN(IO_SIZE, myisam_block_size);
     {
       HA_KEYSEG *pos=share->keyparts;
       uint32 ftkey_nr= 1;
@@ -517,7 +517,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
     share->base.margin_key_file_length=(share->base.max_key_file_length -
 					(keys ? MI_INDEX_BLOCK_MARGIN *
 					 share->blocksize * keys : 0));
-    share->blocksize=min(IO_SIZE,myisam_block_size);
+    share->blocksize= MY_MIN(IO_SIZE, myisam_block_size);
     share->data_file_type=STATIC_RECORD;
     if (share->options & HA_OPTION_COMPRESS_RECORD)
     {
@@ -526,9 +526,9 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
       info.s=share;
       if (_mi_read_pack_info(&info,
 			     (pbool)
-			     test(!(share->options &
-				    (HA_OPTION_PACK_RECORD |
-				     HA_OPTION_TEMP_COMPRESS_RECORD)))))
+			     MY_TEST(!(share->options &
+                                       (HA_OPTION_PACK_RECORD |
+                                        HA_OPTION_TEMP_COMPRESS_RECORD)))))
 	goto err;
     }
     else if (share->options & HA_OPTION_PACK_RECORD)
@@ -595,6 +595,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
 		       &info.buff,(share->base.max_key_block_length*2+
 				   share->base.max_key_length),
 		       &info.lastkey,share->base.max_key_length*3+1,
+                       &info.rnext_same_key, share->base.max_key_length,
 		       &info.first_mbr_key, share->base.max_key_length,
 		       &info.filename,strlen(name)+1,
 		       &info.rtree_recursion_state,have_rtree ? 1024 : 0,
@@ -609,6 +610,11 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
   memcpy(info.blobs,share->blobs,sizeof(MI_BLOB)*share->base.blobs);
   info.lastkey2=info.lastkey+share->base.max_key_length;
 
+  /*
+    If only mi_rkey is called earlier, rnext_same_key should be set in
+    mi_rnext_same.
+  */
+  info.set_rnext_same_key= FALSE;
   info.s=share;
   info.lastpos= HA_OFFSET_ERROR;
   info.update= (short) (HA_STATE_NEXT_FOUND+HA_STATE_PREV_FOUND);
@@ -655,10 +661,10 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
 
   /* Allocate buffer for one record */
 
-  /* prerequisites: bzero(info) && info->s=share; are met. */
+  /* prerequisites: memset(&info, 0) && info->s=share; are met. */
   if (!mi_alloc_rec_buff(&info, -1, &info.rec_buff))
     goto err;
-  bzero(info.rec_buff, mi_get_rec_buff_len(&info, info.rec_buff));
+  memset(info.rec_buff, 0, mi_get_rec_buff_len(&info, info.rec_buff));
 
   *m_info=info;
   thr_lock_data_init(&share->lock,&m_info->lock,(void*) m_info);
@@ -670,7 +676,7 @@ MI_INFO *mi_open(const char *name, int mode, uint open_flags)
     mysql_mutex_unlock(&THR_LOCK_myisam);
   }
 
-  bzero(info.buff, share->base.max_key_block_length * 2);
+  memset(info.buff, 0, share->base.max_key_block_length * 2);
 
   if (myisam_log_file >= 0)
   {
@@ -732,10 +738,10 @@ uchar *mi_alloc_rec_buff(MI_INFO *info, ulong length, uchar **buf)
     if (length == (ulong) -1)
     {
       if (info->s->options & HA_OPTION_COMPRESS_RECORD)
-        length= max(info->s->base.pack_reclength, info->s->max_pack_length);
+        length= MY_MAX(info->s->base.pack_reclength, info->s->max_pack_length);
       else
         length= info->s->base.pack_reclength;
-      length= max(length, info->s->base.max_key_length);
+      length= MY_MAX(length, info->s->base.max_key_length);
       /* Avoid unnecessary realloc */
       if (newptr && length == old_length)
 	return newptr;
@@ -1055,7 +1061,7 @@ uint mi_base_info_write(File file, MI_BASE_INFO *base)
   mi_int2store(ptr,base->max_key_length);		ptr +=2;
   mi_int2store(ptr,base->extra_alloc_bytes);		ptr +=2;
   *ptr++= base->extra_alloc_procent;
-  bzero(ptr,13);					ptr +=13; /* extra */
+  memset(ptr, 0, 13);					ptr +=13; /* extra */
   return mysql_file_write(file, buff, (size_t) (ptr-buff), MYF(MY_NABP)) != 0;
 }
 
@@ -1239,7 +1245,7 @@ exist a dup()-like call that would give us two different file descriptors.
 *************************************************************************/
 
 int mi_open_datafile(MI_INFO *info, MYISAM_SHARE *share, const char *org_name,
-                     File file_to_dup __attribute__((unused)))
+                     File file_to_dup MY_ATTRIBUTE((unused)))
 {
   char *data_name= share->data_file_name;
   char real_data_name[FN_REFLEN];
