@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -347,7 +347,7 @@ CPP_UNNAMED_NS_END
   and adds a child list of TABLE_LIST to the parent handler.
 */
 
-int ha_myisammrg::open(const char *name, int mode __attribute__((unused)),
+int ha_myisammrg::open(const char *name, int mode MY_ATTRIBUTE((unused)),
                        uint test_if_locked_arg)
 {
   DBUG_ENTER("ha_myisammrg::open");
@@ -1183,8 +1183,8 @@ int ha_myisammrg::index_last(uchar * buf)
 }
 
 int ha_myisammrg::index_next_same(uchar * buf,
-                                  const uchar *key __attribute__((unused)),
-                                  uint length __attribute__((unused)))
+                                  const uchar *key MY_ATTRIBUTE((unused)),
+                                  uint length MY_ATTRIBUTE((unused)))
 {
   int error;
   DBUG_ASSERT(this->file->children_attached);
@@ -1509,7 +1509,7 @@ err:
 }
 
 
-int ha_myisammrg::create(const char *name, register TABLE *form,
+int ha_myisammrg::create(const char *name, TABLE *form,
 			 HA_CREATE_INFO *create_info)
 {
   char buff[FN_REFLEN];
